"""
This module downloads the U.S. total nonfarm payrolls seasonally adjusted
(PAYEMS) monthly time series from the St. Louis Federal Reserve's FRED system
(https://fred.stlouisfed.org/series/PAYEMS) or loads it from this directory and
organizes it into 15 series, one for each of the last 15 recessions--from the
current 2020 Coronavirus recession to the Great Depression of 1929. It then
creates a normalized peak plot of the PAYEMS data for each of the last 15
recessions using the Bokeh plotting library.

This module defines the following function(s):
    get_usempl_data()
    usempl_npp()
"""

# Import packages
import numpy as np
import pandas as pd
import pandas_datareader as pddr
import datetime as dt
import os
from bokeh.io import output_file
from bokeh.plotting import figure, show
from bokeh.models import ColumnDataSource, Title, Legend, HoverTool

# from bokeh.models import Label
from bokeh.palettes import Category20

"""
Define functions
"""


def get_usempl_data(
    frwd_mths_max=135,
    bkwd_mths_max=48,
    end_date_str="2022-12-15",
    download_from_internet=True,
):
    """
    This function either downloads or reads in the U.S. total nonfarm payrolls
    seasonally adjusted monthly data series (PAYEMS) and adds variables
    mths_frm_peak and empl_dv_pk for each of the last 15 recessions.

    Args:
        frwd_mths_max (int): maximum number of months forward from the peak
            month to plot
        bckwd_mths_max (int): maximum number of months backward from the peak
            month to plot
        end_date_str (str): end date of PAYEMS time series in 'YYYY-mm-dd'
            format
        download_from_internet (bool): =True if download data from
            fred.stlouisfed.org, otherwise read data in from local directory

    Other functions and files called by this function:
        usempl_[yyyy-mm-dd].csv

    Files created by this function:
        usempl_[yyyy-mm-dd].csv
        usempl_pk_[yyyy-mm-dd].csv

    Returns:
        usempl_pk (DataFrame): N x 46 DataFrame of mths_frm_peak, Date{i},
            Close{i}, and close_dv_pk{i} for each of the 15 recessions for the
            periods specified by bkwd_days_max and frwd_days_max
        end_date_str2 (str): actual end date of DJIA time series in
            'YYYY-mm-dd' format. Can differ from the end_date input to this
            function if the final data for that day have not come out yet
            (usually 2 hours after markets close, 6:30pm EST), or if the
            end_date is one on which markets are closed (e.g. weekends and
            holidays). In this latter case, the pandas_datareader library
            chooses the most recent date for which we have DJIA data.
        peak_vals (list): list of peak DJIA value at the beginning of each of
            the last 15 recessions
        peak_dates (list): list of string date (YYYY-mm-dd) of peak DJIA value
            at the beginning of each of the last 15 recessions
        rec_label_yr_lst (list): list of string start year and end year of each
            of the last 15 recessions
        rec_label_yrmth_lst (list): list of string start year and month and end
            year and month of each of the last 15 recessions
        rec_beg_yrmth_lst (list): list of string start year and month of each
            of the last 15 recessions
        maxdate_rng_lst (list): list of tuples with start string date and end
            string date within which range we define the peak DJIA value at the
            beginning of each of the last 15 recessions
    """
    end_date = dt.datetime.strptime(end_date_str, "%Y-%m-%d")

    # Name the current directory and make sure it has a data folder
    cur_path = os.path.split(os.path.abspath(__file__))[0]
    data_fldr = "data"
    data_dir = os.path.join(cur_path, data_fldr)
    if not os.access(data_dir, os.F_OK):
        os.makedirs(data_dir)

    filename_basic = "usempl_" + end_date_str + ".csv"
    filename_full = "usempl_pk_" + end_date_str + ".csv"

    if download_from_internet:
        # Download the employment data directly from fred.stlouisfed.org
        # (requires internet connection)
        start_date = dt.datetime(1939, 1, 1)
        usempl_df = pddr.fred.FredReader(
            symbols="PAYEMS", start=start_date, end=end_date
        ).read()
        usempl_df = pd.DataFrame(usempl_df).sort_index()  # Sort old to new
        usempl_df = usempl_df.reset_index(level=["DATE"])
        usempl_df = usempl_df.rename(columns={"DATE": "Date"})
        end_date_str2 = usempl_df["Date"].iloc[-1].strftime("%Y-%m-%d")
        end_date = dt.datetime.strptime(end_date_str2, "%Y-%m-%d")
        filename_basic = "usempl_" + end_date_str2 + ".csv"
        filename_full = "usempl_pk_" + end_date_str2 + ".csv"
        usempl_df.to_csv(os.path.join(data_dir, filename_basic), index=False)
        # Merge in U.S. annual average nonfarm payroll employment (not
        # seasonally adjusted) 1919-1938. Date values for annual data are set
        # to July 1 of that year. These data are taken from Table 1 on page 1
        # of "Employment, Hours, and Earnings, United States, 1909-90, Volume
        # I," Bulletin of the United States Bureau of Labor Statistics, No.
        # 2370, March 1991.
        # <https://fraser.stlouisfed.org/title/employment-earnings-united-
        # states-189/employment-hours-earnings-united-states-1909-90-5435/
        # content/pdf/emp_bmark_1909_1990_v1>
        filename_annual = "usempl_annual_1919-1938.csv"
        ann_data_file_path = os.path.join(data_dir, filename_annual)
        usempl_ann_df = pd.read_csv(
            ann_data_file_path,
            names=["Date", "PAYEMS"],
            parse_dates=["Date"],
            skiprows=1,
            na_values=[".", "na", "NaN"],
        )
        # usempl_df = usempl_df.append(usempl_ann_df, ignore_index=True)
        usempl_df = pd.concat([usempl_ann_df, usempl_df], ignore_index=True)
        usempl_df = usempl_df.sort_values(by="Date")
        usempl_df = usempl_df.reset_index(drop=True)
        usempl_df.to_csv(os.path.join(data_dir, filename_basic), index=False)
        # Add other months to annual data 1919-01-01 to 1938-12-01 and fill in
        # artificial employment data by cubic spline interpolation
        months_df = pd.DataFrame(
            pd.date_range("1919-01-01", "1938-12-01", freq="MS"),
            columns=["Date"],
        )
        usempl_df = pd.merge(
            usempl_df, months_df, left_on="Date", right_on="Date", how="outer"
        )
        usempl_df = usempl_df.sort_values(by="Date")
        usempl_df = usempl_df.reset_index(drop=True)
        usempl_df["PAYEMS"].iloc[:242] = (
            usempl_df["PAYEMS"].iloc[:242].interpolate(method="cubic")
        )
    else:
        # Import the data as pandas DataFrame
        end_date_str2 = end_date_str
        data_file_path = os.path.join(data_dir, filename_basic)
        usempl_df = pd.read_csv(
            data_file_path,
            names=["Date", "PAYEMS"],
            parse_dates=["Date"],
            skiprows=1,
            na_values=[".", "na", "NaN"],
        )
        usempl_df = usempl_df.dropna()

    print(
        "End date of U.S. employment series is", end_date.strftime("%Y-%m-%d")
    )

    # Set recession-specific parameters
    rec_label_yr_lst = [
        "1929-1933",  # (Aug 1929 - Mar 1933) Great Depression
        "1937-1938",  # (May 1937 - Jun 1938)
        "1945",  # (Feb 1945 - Oct 1945)
        "1948-1949",  # (Nov 1948 - Oct 1949)
        "1953-1954",  # (Jul 1953 - May 1954)
        "1957-1958",  # (Aug 1957 - Apr 1958)
        "1960-1961",  # (Apr 1960 - Feb 1961)
        "1969-1970",  # (Dec 1969 - Nov 1970)
        "1973-1975",  # (Nov 1973 - Mar 1975)
        "1980",  # (Jan 1980 - Jul 1980)
        "1981-1982",  # (Jul 1981 - Nov 1982)
        "1990-1991",  # (Jul 1990 - Mar 1991)
        "2001",  # (Mar 2001 - Nov 2001)
        "2007-2009",  # (Dec 2007 - Jun 2009) Great Recession
        "2020-2020",
    ]  # (Feb 2020 - Apr 2020) Coronavirus recession

    rec_label_yrmth_lst = [
        "Aug 1929 - Mar 1933",  # Great Depression
        "May 1937 - Jun 1938",
        "Feb 1945 - Oct 1945",
        "Nov 1948 - Oct 1949",
        "Jul 1953 - May 1954",
        "Aug 1957 - Apr 1958",
        "Apr 1960 - Feb 1961",
        "Dec 1969 - Nov 1970",
        "Nov 1973 - Mar 1975",
        "Jan 1980 - Jul 1980",
        "Jul 1981 - Nov 1982",
        "Jul 1990 - Mar 1991",
        "Mar 2001 - Nov 2001",
        "Dec 2007 - Jun 2009",  # Great Recession
        "Feb 2020 - Apr 2020",
    ]  # Coronavirus recess'n

    rec_beg_yrmth_lst = [
        "Aug 1929",
        "May 1937",
        "Feb 1945",
        "Nov 1948",
        "Jul 1953",
        "Aug 1957",
        "Apr 1960",
        "Dec 1969",
        "Nov 1973",
        "Jan 1980",
        "Jul 1981",
        "Jul 1990",
        "Mar 2001",
        "Dec 2007",
        "Feb 2020",
    ]

    maxdate_rng_lst = [
        ("1929-7-1", "1929-10-1"),
        ("1937-7-1", "1937-7-1"),
        ("1945-1-1", "1945-3-1"),
        ("1948-9-1", "1949-1-1"),
        ("1953-6-1", "1953-8-1"),
        ("1957-7-1", "1957-9-1"),
        ("1960-3-1", "1960-5-1"),
        ("1969-11-1", "1970-3-1"),
        ("1973-10-1", "1974-7-1"),
        ("1979-12-1", "1980-3-1"),
        ("1981-6-1", "1981-8-1"),
        ("1990-6-1", "1991-8-1"),
        ("2001-2-1", "2001-4-1"),
        ("2007-11-1", "2008-1-1"),
        ("2020-1-1", "2020-3-1"),
    ]

    # Create normalized peak series for each recession
    usempl_pk = pd.DataFrame(
        np.arange(-bkwd_mths_max, frwd_mths_max + 1, dtype=int),
        columns=["mths_frm_peak"],
    )
    usempl_pk_long = usempl_df.copy()
    peak_vals = []
    peak_dates = []
    for i, maxdate_rng in enumerate(maxdate_rng_lst):
        # Identify peak closing value within two months (with only 2
        # exceptions) of the beginning month of the recession
        peak_val = usempl_df["PAYEMS"][
            (usempl_df["Date"] >= maxdate_rng[0])
            & (usempl_df["Date"] <= maxdate_rng[1])
        ].max()
        peak_vals.append(peak_val)
        usempl_dv_pk_name = "usempl_dv_pk" + str(i)
        usempl_pk_long[usempl_dv_pk_name] = usempl_pk_long["PAYEMS"] / peak_val
        # Identify date of peak PAYEMS value within two months (with
        # only 2 exceptions) of the beginning month of the recession
        peak_date = usempl_df["Date"][
            (usempl_df["Date"] >= maxdate_rng[0])
            & (usempl_df["Date"] <= maxdate_rng[1])
            & (usempl_df["PAYEMS"] == peak_val)
        ].max()
        peak_dates.append(peak_date.strftime("%Y-%m-%d"))
        mths_frm_pk_name = "mths_frm_pk" + str(i)
        usempl_pk_long[mths_frm_pk_name] = (
            usempl_pk_long["Date"].dt.year - peak_date.year
        ) * 12 + (usempl_pk_long["Date"].dt.month - peak_date.month)
        # usempl_pk_long[mths_frm_pk_name] = (usempl_pk_long['Date'] -
        #                                     peak_date).dt.years
        print(
            "peak_val " + str(i) + " is",
            peak_val,
            "on date",
            peak_date.strftime("%Y-%m-%d"),
            "(Beg. rec. month:",
            rec_beg_yrmth_lst[i],
            ")",
        )
        # I need to merge the data into this new usempl_pk DataFrame so that
        # mths_frm_peak variable is shared across the dataframe
        usempl_pk = pd.merge(
            usempl_pk,
            usempl_pk_long[
                [mths_frm_pk_name, "Date", "PAYEMS", usempl_dv_pk_name]
            ],
            left_on="mths_frm_peak",
            right_on=mths_frm_pk_name,
            how="left",
        )
        usempl_pk.drop(columns=[mths_frm_pk_name], inplace=True)
        usempl_pk.rename(
            columns={"Date": f"Date{i}", "PAYEMS": f"PAYEMS{i}"}, inplace=True
        )

    usempl_pk.to_csv(os.path.join(data_dir, filename_full), index=False)

    return (
        usempl_pk,
        end_date_str2,
        peak_vals,
        peak_dates,
        rec_label_yr_lst,
        rec_label_yrmth_lst,
        rec_beg_yrmth_lst,
        maxdate_rng_lst,
    )


def usempl_npp(
    frwd_mths_main=85,
    bkwd_mths_main=7,
    frwd_mths_max=135,
    bkwd_mths_max=48,
    usempl_end_date="today",
    download_from_internet=True,
    html_show=True,
):
    """
    This function creates the HTML and JavaScript code for the dynamic
    visualization of the normalized peak plot of the last 15 recessions in the
    United States, from the Great Depression (Aug. 1929 - Mar. 1933) to the
    most recent COVID-19 recession (Feb. 2020 - present).

    Args:
        frwd_mths_main (int): number of months forward from the peak to plot in
            the default main window of the visualization
        bkwd_mths_maim (int): number of months backward from the peak to plot
            in the default main window of the visualization
        frwd_mths_max (int): maximum number of months forward from the peak to
            allow for the plot, to be seen by zooming out
        bkwd_mths_max (int): maximum number of months backward from the peak to
            allow for the plot, to be seen by zooming out
        usempl_end_date (str): either 'today' or the end date of PAYEMS time
            series in 'YYYY-mm-dd' format
        download_from_internet (bool): =True if download data from St. Louis
            Federal Reserve's FRED system
            (https://fred.stlouisfed.org/series/PAYEMS), otherwise read data in
            from local directory
        html_show (bool): =True if open dynamic visualization in browser once
            created

    Other functions and files called by this function:
        get_usempl_data()

    Files created by this function:
       images/usempl_[yyyy-mm-dd].html

    Returns: fig, end_date_str
    """
    # Create directory if images directory does not already exist
    cur_path = os.path.split(os.path.abspath(__file__))[0]
    image_fldr = "images"
    image_dir = os.path.join(cur_path, image_fldr)
    if not os.access(image_dir, os.F_OK):
        os.makedirs(image_dir)

    if usempl_end_date == "today":
        end_date = dt.date.today()  # Go through today
    else:
        end_date = dt.datetime.strptime(usempl_end_date, "%Y-%m-%d")

    end_date_str = end_date.strftime("%Y-%m-%d")

    # Set main window and total data limits for monthly plot
    frwd_mths_main = int(frwd_mths_main)
    bkwd_mths_main = int(bkwd_mths_main)
    frwd_mths_max = int(frwd_mths_max)
    bkwd_mths_max = int(bkwd_mths_max)

    (
        usempl_pk,
        end_date_str2,
        peak_vals,
        peak_dates,
        rec_label_yr_lst,
        rec_label_yrmth_lst,
        rec_beg_yrmth_lst,
        maxdate_rng_lst,
    ) = get_usempl_data(
        frwd_mths_max, bkwd_mths_max, end_date_str, download_from_internet
    )
    if end_date_str2 != end_date_str:
        print(
            "PAYEMS data downloaded on "
            + end_date_str
            + " has most "
            + "recent PAYEMS data month of "
            + end_date_str2
            + "."
        )
    end_date2 = dt.datetime.strptime(end_date_str2, "%Y-%m-%d")

    rec_cds_list = []
    min_main_val_lst = []
    max_main_val_lst = []
    for i in range(15):
        usempl_pk_rec = usempl_pk[
            ["mths_frm_peak", f"Date{i}", f"PAYEMS{i}", f"usempl_dv_pk{i}"]
        ].dropna()
        usempl_pk_rec.rename(
            columns={
                f"Date{i}": "Date",
                f"PAYEMS{i}": "PAYEMS",
                f"usempl_dv_pk{i}": "usempl_dv_pk",
            },
            inplace=True,
        )
        rec_cds_list.append(ColumnDataSource(usempl_pk_rec))
        # Find minimum and maximum usempl_dv_pk values as inputs to main plot
        # frame size
        min_main_val_lst.append(
            usempl_pk_rec["usempl_dv_pk"][
                (usempl_pk_rec["mths_frm_peak"] >= -bkwd_mths_main)
                & (usempl_pk_rec["mths_frm_peak"] <= frwd_mths_main)
            ].min()
        )
        max_main_val_lst.append(
            usempl_pk_rec["usempl_dv_pk"][
                (usempl_pk_rec["mths_frm_peak"] >= -bkwd_mths_main)
                & (usempl_pk_rec["mths_frm_peak"] <= frwd_mths_main)
            ].max()
        )

    # Create Bokeh plot of PAYEMS normalized peak plot figure
    fig_title = "Progression of PAYEMS in last 15 recessions"
    filename = "usempl_npp_" + end_date_str2 + ".html"
    output_file(os.path.join(image_dir, filename), title=fig_title)

    # Format the tooltip
    tooltips = [
        ("Date", "@Date{%F}"),
        ("Months from peak", "$x{0.}"),
        ("Employment", "@PAYEMS{0,0.},000"),
        ("Fraction of peak", "@usempl_dv_pk{0.0 %}"),
    ]

    # Solve for minimum and maximum PAYEMS/Peak values in monthly main display
    # window in order to set the appropriate xrange and yrange
    min_main_val = min(min_main_val_lst)
    max_main_val = max(max_main_val_lst)

    datarange_main_vals = max_main_val - min_main_val
    datarange_main_mths = int(frwd_mths_main + bkwd_mths_main)
    fig_buffer_pct = 0.10
    fig = figure(
        height=450,
        width=800,
        x_axis_label="Months from peak",
        y_axis_label="Employment as fraction of peak",
        y_range=(
            min_main_val - fig_buffer_pct * datarange_main_vals,
            max_main_val + fig_buffer_pct * datarange_main_vals,
        ),
        x_range=(
            (-bkwd_mths_main - fig_buffer_pct * datarange_main_mths),
            (frwd_mths_main + fig_buffer_pct * datarange_main_mths),
        ),
        tools=[
            "save",
            "zoom_in",
            "zoom_out",
            "box_zoom",
            "pan",
            "undo",
            "redo",
            "reset",
            "help",
        ],
        toolbar_location="left",
    )
    fig.toolbar.logo = None

    l0 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[0],
        color="blue",
        line_width=5,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l1 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[1],
        color=Category20[13][0],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l2 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[2],
        color=Category20[13][1],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l3 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[3],
        color=Category20[13][2],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l4 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[4],
        color=Category20[13][3],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l5 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[5],
        color=Category20[13][4],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l6 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[6],
        color=Category20[13][5],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l7 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[7],
        color=Category20[13][6],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l8 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[8],
        color=Category20[13][7],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l9 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[9],
        color=Category20[13][8],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l10 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[10],
        color=Category20[13][9],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l11 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[11],
        color=Category20[13][10],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l12 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[12],
        color=Category20[13][11],
        line_width=2,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l13 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[13],
        color=Category20[13][12],
        line_width=5,
        alpha=0.7,
        muted_alpha=0.15,
    )
    l14 = fig.line(
        x="mths_frm_peak",
        y="usempl_dv_pk",
        source=rec_cds_list[14],
        color="black",
        line_width=5,
        alpha=0.7,
        muted_alpha=0.15,
    )

    # Dashed vertical line at the peak PAYEMS value period
    fig.line(
        x=[0.0, 0.0],
        y=[-0.5, 2.0],
        color="black",
        line_width=2,
        line_dash="dashed",
        alpha=0.5,
    )

    # Dashed horizontal line at PAYEMS as fraction of peak equals 1
    fig.line(
        x=[-bkwd_mths_max, frwd_mths_max],
        y=[1.0, 1.0],
        color="black",
        line_width=2,
        line_dash="dashed",
        alpha=0.5,
    )

    # # Create the tick marks for the x-axis and set x-axis labels
    # major_tick_labels = []
    # major_tick_list = []
    # for i in range(-bkwd_mths_max, frwd_mths_max + 1):
    #     if i % 2 == 0:  # indicates even integer
    #         major_tick_list.append(int(i))
    #         if i < 0:
    #             major_tick_labels.append(str(i) + 'mth')
    #         elif i == 0:
    #             major_tick_labels.append('peak')
    #         elif i > 0:
    #             major_tick_labels.append('+' + str(i) + 'mth')

    # # minor_tick_list = [item for item in range(-bkwd_mths_max,
    # #                                           frwd_mths_max + 1)]
    # major_tick_dict = dict(zip(major_tick_list, major_tick_labels))
    # fig.xaxis.ticker = major_tick_list
    # fig.xaxis.major_label_overrides = major_tick_dict

    # Add legend
    legend = Legend(
        items=[
            (rec_label_yrmth_lst[0], [l0]),
            (rec_label_yrmth_lst[1], [l1]),
            (rec_label_yrmth_lst[2], [l2]),
            (rec_label_yrmth_lst[3], [l3]),
            (rec_label_yrmth_lst[4], [l4]),
            (rec_label_yrmth_lst[5], [l5]),
            (rec_label_yrmth_lst[6], [l6]),
            (rec_label_yrmth_lst[7], [l7]),
            (rec_label_yrmth_lst[8], [l8]),
            (rec_label_yrmth_lst[9], [l9]),
            (rec_label_yrmth_lst[10], [l10]),
            (rec_label_yrmth_lst[11], [l11]),
            (rec_label_yrmth_lst[12], [l12]),
            (rec_label_yrmth_lst[13], [l13]),
            (rec_label_yrmth_lst[14], [l14]),
        ],
        location="center",
    )
    fig.add_layout(legend, "right")

    # # Add label to current recession low point
    # fig.text(x=[12, 12, 12, 12], y=[0.63, 0.60, 0.57, 0.54],
    #          text=['2020-03-23', 'DJIA: 18,591.93', '63.3% of peak',
    #                '39 days from peak'],
    #          text_font_size='8pt', angle=0)

    # label_text = ('Recent low \n 2020-03-23 \n DJIA: 18,591.93 \n '
    #               '63\% of peak \n 39 days from peak')
    # fig.add_layout(Label(x=10, y=0.65, x_units='screen', text=label_text,
    #                      render_mode='css', border_line_color='black',
    #                      border_line_alpha=1.0,
    #                      background_fill_color='white',
    #                      background_fill_alpha=1.0))

    # Add title and subtitle to the plot
    fig_title2 = "US nonfarm employment, normalized peak plot,"
    fig_title3 = "comparison of last 15 recessions, 1929 to 2024"
    fig.add_layout(
        Title(
            text=fig_title3,
            text_font_style="bold",
            text_font_size="15pt",
            align="center",
        ),
        "above",
    )
    fig.add_layout(
        Title(
            text=fig_title2,
            text_font_style="bold",
            text_font_size="15pt",
            align="center",
        ),
        "above",
    )

    # Add source text below figure
    updated_date_str = (
        end_date.strftime("%B")
        + " "
        + end_date.strftime("%d").lstrip("0")
        + ", "
        + end_date.strftime("%Y")
    )
    fig.add_layout(
        Title(
            text="Source: Richard W. Evans (@RickEcon), "
            + "historical PAYEMS data from FRED and BLS, "
            + "updated "
            + updated_date_str
            + ".",
            align="left",
            text_font_size="3mm",
            text_font_style="italic",
        ),
        "below",
    )
    fig.legend.click_policy = "mute"

    # Add the HoverTool to the figure
    fig.add_tools(
        HoverTool(
            tooltips=tooltips,
            toggleable=False,
            formatters={"@Date": "datetime"},
        )
    )

    if html_show:
        show(fig)

    return fig, end_date_str


if __name__ == "__main__":
    # execute only if run as a script
    fig, end_date_str = usempl_npp()



# Import packages
import numpy as np
import pandas as pd
import datetime as dt
import os
from usempl_plots.get_payems import get_payems_data

from bokeh.core.property.numeric import Interval
from bokeh.models.annotations import Label, LabelSet
from bokeh.models.glyphs import VArea
from bokeh.models.tickers import SingleIntervalTicker
from bokeh.io import output_file
from bokeh.plotting import figure, show
from bokeh.models import (
    ColumnDataSource,
    Title,
    Legend,
    HoverTool,
    NumeralTickFormatter,
)

"""
-------------------------------------------------------------------------------
Create pandas DataFrames and Column Data Source data objects
-------------------------------------------------------------------------------
"""


def gen_payems_tseries(
    start_date="min",
    end_date="max",
    recession_bars=True,
    download=True,
    fig_title_str=(
        "Time Series of US Monthly Nonfarm Payroll Employment (PAYEMS)"
    ),
    html_show=True,
):
    """
    This function creates a simple time series plot of US nonfarm payroll
    employment (PAYEMS).

    Args:
        start_date (str): start date of PAYEMS time series in 'YYYY-mm-dd'
            format or 'min'
    """
    # Create data and images directory as well as recession data path
    cur_path = os.path.split(os.path.abspath(__file__))[0]
    image_dir = os.path.join(cur_path, "images")
    data_dir = os.path.join(cur_path, "data")
    recession_data_path = os.path.join(data_dir, "recession_data.csv")

    # Get the employment data
    if start_date == "min":
        beg_date_str = "1919-01-01"
    else:
        try:
            beg_date_test = dt.datetime.strptime(start_date, "%Y-%m-%d")
        except:
            err_msg = (
                "Error get_payems.py: start_date input must be either a "
                + "date string in 'YYYY-mm-dd' format or 'min'."
            )
            raise ValueError(err_msg)
        beg_date_str = start_date

    if end_date == "max":
        end_date_str = "today"
    else:
        try:
            end_date_test = dt.datetime.strptime(end_date, "%Y-%m-%d")
        except:
            err_msg = (
                "Error get_payems.py: end_date input must be either a "
                + "date string in 'YYYY-mm-dd' format or 'max'."
            )
            raise ValueError(err_msg)
        end_date_str = end_date

    if end_date_str == "today":
        download_date = dt.datetime.today()
    else:
        download_date = dt.datetime.strptime(end_date_str, "%Y-%m-%d")
    download_date_str = download_date.strftime("%Y-%m-%d")
    if download:
        usempl_df, beg_date_str2, end_date_str2 = get_payems_data(
            beg_date_str=beg_date_str,
            end_date_str=end_date_str,
            file_path=None,
        )
        print(
            "PAYEMS data downloaded on "
            + download_date_str
            + " and has most recent PAYEMS data month of "
            + end_date_str2
            + "."
        )
    else:
        usempl_df, beg_date_str2, end_date_str2 = get_payems_data(
            beg_date_str=beg_date_str,
            end_date_str=end_date_str,
            file_path=os.path.join(
                data_dir, "usempl_" + end_date_str + ".csv"
            ),
        )
        print(
            "PAYEMS data loaded from memory on "
            + download_date_str
            + " and has most recent PAYEMS data month of "
            + end_date_str2
            + "."
        )

    # Create a dataframe that only contains dates less than 1939-01-01
    usempl_imputed_df = usempl_df[usempl_df["Date"] < "1939-01-01"]
    usempl_monthly_df = usempl_df[usempl_df["Date"] >= "1939-01-01"]
    # Create a dataframe usempl_annual_df that only contains data from before
    # 1939 and only for month 7
    usempl_annual_df = usempl_imputed_df[
        usempl_imputed_df["Date"].dt.month == 7
    ]
    usempl_imputed_cds = ColumnDataSource(usempl_imputed_df)
    usempl_annual_cds = ColumnDataSource(usempl_annual_df)
    usempl_monthly_cds = ColumnDataSource(usempl_monthly_df)
    # print(usempl_monthly_df.keys())
    # print(usempl_annual_df.keys())
    # print(usempl_imputed_df.keys())
    # print(usempl_annual_df[['Date', 'PAYEMS', 'BLS_annual']])

    # Create recession data column data source object
    recession_df = pd.read_csv(
        recession_data_path, parse_dates=["Peak", "Trough"]
    )
    recession_data_length = len(recession_df["Peak"])

    # Create Bokeh plot of PAYEMS time series
    fig_title = fig_title_str
    filename = "tseries_payems_" + end_date_str2 + ".html"
    output_file(
        os.path.join(image_dir, filename), title=fig_title, mode="inline"
    )

    # Format the tooltip
    tooltips = [
        ("Date", "@Date{%F}"),
        ("Employed", "@PAYEMS{0,0.}"),
        ("Change from prev. month", "@diff_monthly{0,0.}"),
        ("Change from prev. year", "@diff_yoy{0,0.}"),
    ]

    min_date = usempl_df["Date"].min()
    max_date = usempl_df["Date"].max()
    min_y_val = usempl_df["PAYEMS"].min()
    max_y_val = usempl_df["PAYEMS"].max()
    range_y_vals = max_y_val - min_y_val
    fig_buffer_pct = 0.10
    fig = figure(
        height=400,
        width=800,
        x_axis_label="Date (monthly)",
        y_axis_label="Employment (millions)",
        y_range=(
            min_y_val - fig_buffer_pct * range_y_vals,
            max_y_val + fig_buffer_pct * range_y_vals,
        ),
        y_minor_ticks=2,
        x_range=(min_date, max_date),
        x_minor_ticks=2,
        tools=[
            "save",
            "zoom_in",
            "zoom_out",
            "box_zoom",
            "pan",
            "undo",
            "redo",
            "reset",
            "help",
        ],
        toolbar_location="left",
    )
    fig.toolbar.logo = None

    # Set title font size and axes font sizes
    fig.xaxis.axis_label_text_font_size = "12pt"
    fig.xaxis.major_label_text_font_size = "12pt"
    fig.yaxis.axis_label_text_font_size = "12pt"
    fig.yaxis.major_label_text_font_size = "12pt"

    # Reformat the labels for the ticks on the x and y axes
    y_tick_label_dict = {
        20_000_000: "20m",
        40_000_000: "40m",
        60_000_000: "60m",
        80_000_000: "80m",
        100_000_000: "100m",
        120_000_000: "120m",
        140_000_000: "140m",
        160_000_000: "160m",
    }

    fig.yaxis.major_label_overrides = y_tick_label_dict

    monthly = fig.line(
        x="Date",
        y="PAYEMS",
        source=usempl_monthly_cds,
        color="blue",
        line_dash="solid",
        line_width=3,
        alpha=0.7,
        muted_alpha=0.1,
        legend_label="Monthly data",
    )

    imputed = fig.line(
        x="Date",
        y="PAYEMS",
        source=usempl_imputed_cds,
        color="red",
        line_dash="solid",
        line_width=3,
        alpha=0.7,
        muted_alpha=0.1,
        legend_label="Interpolated from annual data",
    )

    annual = fig.scatter(
        x="Date",
        y="PAYEMS",
        source=usempl_annual_cds,
        size=2,
        line_width=1,
        line_color="black",
        fill_color="purple",
        alpha=0.7,
        muted_alpha=0.1,
        legend_label="Annual data points",
    )

    if recession_bars:
        # Create recession bars
        for x in range(0, recession_data_length):
            peak_date = recession_df["Peak"][x]
            trough_date = recession_df["Trough"][x]
            if peak_date >= min_date and trough_date >= min_date:
                fig.patch(
                    x=[peak_date, trough_date, trough_date, peak_date],
                    y=[
                        -10_000_000,
                        -10_000_000,
                        max_y_val + 100_000_000,
                        max_y_val + 100_000_000,
                    ],
                    fill_color="gray",
                    fill_alpha=0.4,
                    line_width=0,
                    legend_label="Recession",
                )
            if (
                peak_date == trough_date
                and peak_date >= min_date
                and trough_date >= min_date
            ):
                fig.patch(
                    x=[peak_date, trough_date, trough_date, peak_date],
                    y=[
                        -10_000_000,
                        -10_000_000,
                        max_y_val + 100_000_000,
                        max_y_val + 100_000_000,
                    ],
                    fill_color="gray",
                    fill_alpha=0.4,
                    line_width=0,
                    legend_label="Recession",
                )

    hover = HoverTool(
        tooltips=tooltips,
        visible=False,
        formatters={"@Date": "datetime"},
    )
    hover.renderers = [monthly, imputed]

    # Add the HoverTool to the figure
    fig.add_tools(hover)

    # Add legend
    fig.legend.location = "top_left"
    fig.legend.border_line_width = 2
    fig.legend.border_line_color = "black"
    fig.legend.border_line_alpha = 1
    fig.legend.label_text_font_size = "4mm"

    # Set legend muting click policy
    fig.legend.click_policy = "mute"

    # Add title
    fig.add_layout(
        Title(
            text=fig_title_str,
            text_font_style="bold",
            text_font_size="15pt",
            align="center",
        ),
        "above",
    )

    # Add notes below image. The list note_text_list contains a tuple with a
    # string for every line of the notes
    updated_date_str = (
        download_date.strftime("%B")
        + " "
        + download_date.strftime("%d").lstrip("0")
        + ", "
        + download_date.strftime("%Y")
    )
    note_text_list = [
        (
            "Source: Richard W. Evans (@RickEcon), historical PAYEMS data "
            + "from FRED and BLS, updated "
        ),
        ("   " + updated_date_str + "."),
    ]
    for note_text in note_text_list:
        caption = Title(
            text=note_text,
            align="left",
            text_font_size="4mm",
            text_font_style="italic",
        )
        fig.add_layout(caption, "below")

    if html_show:
        show(fig)

    return fig, beg_date_str2, end_date_str2


if __name__ == "__main__":
    # execute only if run as a script
    fig, beg_date_str, end_date_str = gen_payems_tseries()



"""
This module plots the streaks of consecutive positive employment gains from the
US nonfarm employment (PAYEMS) seasonally adjusted monthly time series from the
St. Louis Federal Reserve's FRED system
(https://fred.stlouisfed.org/series/PAYEMS) or loads it from this directory and
organizes it into series of consecutive positive monthly gain streaks.

This module defines the following function(s):
    usempl_streaks()
"""

# Import packages
import numpy as np
import pandas as pd
import pandas_datareader as pddr
import datetime as dt
import os
from usempl_plots.get_payems import get_payems_data
from bokeh.io import output_file
from bokeh.plotting import curdoc, figure, show
from bokeh.models import ColumnDataSource, Title, Legend, HoverTool, LabelSet
from bokeh.layouts import gridplot

# from bokeh.models import Label
from bokeh.palettes import Category20, Plasma256, Viridis8

"""
Define functions
"""


def usempl_streaks(
    start_date="min",
    end_date="max",
    download_from_internet=True,
    scatter_histogram=True,
    table_output=True,
    html_show=True,
):
    """
    This function creates the HTML and JavaScript code for the dynamic
    visualization of the US positive employment streaks from 1939 (the
    beginning of US monthly data) to the given end date.

    Args:
        start_date (str): start date of PAYEMS time series in 'YYYY-mm-dd'
            format or 'min'
    """
    # Create data and images directory paths
    cur_path = os.path.split(os.path.abspath(__file__))[0]
    image_dir = os.path.join(cur_path, "images")
    data_dir = os.path.join(cur_path, "data")

    # Get the employment data
    if start_date == "min":
        beg_date_str = "1939-01-01"
    else:
        try:
            beg_date_test = dt.datetime.strptime(start_date, "%Y-%m-%d")
        except:
            err_msg = (
                "Error get_payems.py: start_date input must be either a "
                + "date string in 'YYYY-mm-dd' format or 'min'."
            )
            raise ValueError(err_msg)
        beg_date_str = start_date

    if end_date == "max":
        end_date_str = "today"
    else:
        try:
            end_date_test = dt.datetime.strptime(end_date, "%Y-%m-%d")
        except:
            err_msg = (
                "Error get_payems.py: end_date input must be either a "
                + "date string in 'YYYY-mm-dd' format or 'max'."
            )
            raise ValueError(err_msg)
        end_date_str = end_date

    if end_date_str == "today":
        download_date = dt.datetime.today()
    else:
        download_date = dt.datetime.strptime(end_date_str, "%Y-%m-%d")
    download_date_str = download_date.strftime("%Y-%m-%d")
    if download_from_internet:
        usempl_df, beg_date_str2, end_date_str2 = get_payems_data(
            beg_date_str=beg_date_str,
            end_date_str=end_date_str,
            file_path=None,
        )
        print(
            "PAYEMS data downloaded on "
            + download_date_str
            + " and has most recent PAYEMS data month of "
            + end_date_str2
            + "."
        )
    else:
        usempl_df, beg_date_str2, end_date_str2 = get_payems_data(
            beg_date_str=beg_date_str,
            end_date_str=end_date_str,
            file_path=os.path.join(
                data_dir, "usempl_" + end_date_str + ".csv"
            ),
        )
        print(
            "PAYEMS data loaded from memory on "
            + download_date_str
            + " and has most recent PAYEMS data month of "
            + end_date_str2
            + "."
        )

    strk_df_lst = []
    strk_cds_lst = []
    strk_label_lst = []
    min_gain_val_lst = []
    max_gain_val_lst = []
    min_cum_val_lst = []
    max_cum_val_lst = []
    min_mth_val_lst = []
    max_mth_val_lst = []
    strk_num = 0
    strk_mths = 0
    strk_cum = 0
    strk_table_df = pd.DataFrame(
        columns=[
            "strk_num",
            "start_date",
            "end_date",
            "months_in_streak",
            "total_emp_gain",
            "avg_monthly_emp_gain",
            "marker_color",
        ]
    )
    for i in range(usempl_df.shape[0]):
        if usempl_df.loc[i, "diff_monthly"] > 0:
            if strk_mths == 0:
                j = i
                strk_num += 1
                strk_df = pd.DataFrame(
                    columns=[
                        "Date",
                        "PAYEMS",
                        "diff_monthly",
                        "strk_mths",
                        "strk_cum",
                        "strk_mths_tot",
                        "strk_avg_emp_gain",
                        "strk_cum_tot",
                    ]
                )
                strk_start_mth_str = usempl_df.loc[i, "Date"].strftime("%Y-%m")
            strk_df.loc[i - j, ["Date", "PAYEMS", "diff_monthly"]] = (
                usempl_df.loc[i, ["Date", "PAYEMS", "diff_monthly"]]
            )
            strk_mths += 1
            strk_df.loc[i - j, "strk_mths"] = strk_mths
            strk_cum += usempl_df.loc[i, "diff_monthly"]
            strk_df.loc[i - j, "strk_cum"] = strk_cum
            if i == usempl_df.shape[0] - 1:
                strk_end_mth_str = usempl_df.loc[i, "Date"].strftime("%Y-%m")
                strk_label = strk_start_mth_str + " to " + strk_end_mth_str
                strk_label_lst.append(strk_label)
                strk_df["strk_mths_tot"] = strk_mths
                strk_df["strk_avg_emp_gain"] = strk_cum / strk_mths
                strk_df["strk_cum_tot"] = strk_cum
                strk_df_lst.append(strk_df)
                strk_cds_lst.append(ColumnDataSource(strk_df))
                min_gain_val_lst.append(strk_df["diff_monthly"].min())
                max_gain_val_lst.append(strk_df["diff_monthly"].max())
                min_cum_val_lst.append(strk_df["strk_cum"].min())
                max_cum_val_lst.append(strk_df["strk_cum"].max())
                min_mth_val_lst.append(strk_df["strk_mths"].min())
                max_mth_val_lst.append(strk_df["strk_mths"].max())
                strk_table_df.loc[strk_num - 1] = [
                    strk_num,
                    strk_start_mth_str,
                    strk_end_mth_str,
                    strk_mths,
                    strk_cum,
                    strk_cum / strk_mths,
                    "",
                ]
        elif (
            usempl_df.loc[i, "diff_monthly"] <= 0
            and usempl_df.loc[i - 1, "diff_monthly"] > 0
            and i > 0
        ):
            strk_df["strk_mths_tot"] = strk_mths
            strk_df["strk_avg_emp_gain"] = strk_cum / strk_mths
            strk_df["strk_cum_tot"] = strk_cum
            strk_end_mth_str = usempl_df.loc[i - 1, "Date"].strftime("%Y-%m")
            strk_label = strk_start_mth_str + " to " + strk_end_mth_str
            strk_label_lst.append(strk_label)
            strk_df_lst.append(strk_df)
            strk_cds_lst.append(ColumnDataSource(strk_df))
            min_gain_val_lst.append(strk_df["diff_monthly"].min())
            max_gain_val_lst.append(strk_df["diff_monthly"].max())
            min_cum_val_lst.append(strk_df["strk_cum"].min())
            max_cum_val_lst.append(strk_df["strk_cum"].max())
            min_mth_val_lst.append(strk_df["strk_mths"].min())
            max_mth_val_lst.append(strk_df["strk_mths"].max())
            strk_table_df.loc[strk_num - 1] = [
                strk_num,
                strk_start_mth_str,
                strk_end_mth_str,
                strk_mths,
                strk_cum,
                strk_cum / strk_mths,
                "",
            ]
            strk_mths = 0
            strk_cum = 0

    print("Number of streaks:", strk_num)

    # Create Bokeh plot of PAYEMS normalized peak plot figure
    fig_title_lst = [
        ("US employment streaks: consecutive positive monthly"),
        ("gains and cumulative employment gains, 1939 to 2024"),
    ]
    fig_title_short = "Figure 1. US employment streaks"
    filename_strk = "usempl_streaks_" + end_date_str2 + ".html"
    output_file(
        os.path.join(image_dir, filename_strk),
        title=fig_title_short,
        mode="inline",
    )

    # Format the tooltip
    tooltips = [
        ("Date", "@Date{%F}"),
        ("Current month in streak", "@strk_mths{0.}"),
        ("Total months in streak", "@strk_mths_tot{0.}"),
        ("Monthly employment gain", "@diff_monthly{0,0.}"),
        ("Cumulative employment gain", "@strk_cum{0,0.}"),
        ("Total cumulative employment gain", "@strk_cum_tot{0,0.}"),
        ("Avg. monthly employment gain", "@strk_avg_emp_gain{0,0.}"),
    ]

    # Solve for minimum and maximum PAYEMS/Peak values in monthly main display
    # window in order to set the appropriate xrange and yrange
    min_cum_val = min(min_cum_val_lst)
    max_cum_val = max(max_cum_val_lst)
    min_mth_val = min(min_mth_val_lst)
    max_mth_val = max(max_mth_val_lst)

    datarange_cum_vals = max_cum_val - min_cum_val
    datarange_mth_vals = int(min_mth_val + max_mth_val)
    fig_buffer_pct = 0.05
    fig_strk = figure(
        height=500,
        width=800,
        x_axis_label="Streak length (months)",
        y_axis_label="Cumulative employment gains (millions)",
        y_range=(
            min_cum_val - fig_buffer_pct * datarange_cum_vals,
            max_cum_val + fig_buffer_pct * datarange_cum_vals,
        ),
        y_minor_ticks=5,
        x_range=(
            (min_mth_val - fig_buffer_pct * datarange_mth_vals),
            (max_mth_val + fig_buffer_pct * datarange_mth_vals),
        ),
        tools=[
            "save",
            "zoom_in",
            "zoom_out",
            "box_zoom",
            "pan",
            "undo",
            "redo",
            "reset",
            "hover",
            "help",
        ],
        toolbar_location="left",
    )
    fig_strk.toolbar.logo = None

    # Set title font size and axes font sizes
    fig_strk.xaxis.axis_label_text_font_size = "12pt"
    fig_strk.xaxis.major_label_text_font_size = "12pt"
    fig_strk.yaxis.axis_label_text_font_size = "12pt"
    fig_strk.yaxis.major_label_text_font_size = "12pt"

    # Reformat the labels for the ticks on the x and y axes
    y_tick_label_dict_strk = {
        0: "0",
        5_000_000: "5m",
        10_000_000: "10m",
        15_000_000: "15m",
        20_000_000: "20m",
    }

    fig_strk.yaxis.major_label_overrides = y_tick_label_dict_strk

    fig_lst_strk = []
    label_items_strk_lst = []
    j = -1
    for i in range(strk_num):
        if max_cum_val_lst[i] > 10_000_000 or max_mth_val_lst[i] > 40:
            j += 1
            strk_table_df.loc[strk_table_df.index == i, "marker_color"] = (
                Viridis8[7 - j]
            )
            li = fig_strk.line(
                x="strk_mths",
                y="strk_cum",
                source=strk_cds_lst[i],
                color=Viridis8[7 - j],
                # color=Plasma256[255 - int(i * (256 - 1) / (strk_num - 1))],
                line_width=4,
                alpha=0.7,
                muted_alpha=0.15,
            )
            label_items_strk_lst.append((strk_label_lst[i], [li]))
        else:
            # replace all values of strk_table_df["marker_color"] with "orange" if streak_table_df["months_in_streak"]<40 and streak_table_df["total_emp_gain"]<10_000_000
            strk_table_df.loc[
                (strk_table_df["months_in_streak"] < 40)
                & (strk_table_df["total_emp_gain"] < 10_000_000),
                "marker_color",
            ] = "orange"
            li = fig_strk.line(
                x="strk_mths",
                y="strk_cum",
                source=strk_cds_lst[i],
                color="orange",
                line_width=2,
                alpha=0.7,
                muted_alpha=0.15,
            )
        fig_lst_strk.append(li)

    # Add legend
    legend = Legend(items=label_items_strk_lst, location="center")
    fig_strk.add_layout(legend, "right")
    fig_strk.legend.click_policy = "mute"

    # Add title and subtitle to the plot doing reverse loop through items in
    # fig_title_lst
    for title_line_str in fig_title_lst[::-1]:
        fig_strk.add_layout(
            Title(
                text=title_line_str,
                text_font_style="bold",
                text_font_size="15pt",
                align="center",
            ),
            "above",
        )

    # Add source text below figure
    updated_date_str = (
        download_date.strftime("%B")
        + " "
        + download_date.strftime("%d").lstrip("0")
        + ", "
        + download_date.strftime("%Y")
    )
    note_text_list = [
        (
            "Source: Richard W. Evans (@RickEcon), historical PAYEMS data "
            + "from FRED and BLS, updated "
        ),
        ("    " + updated_date_str + "."),
    ]
    for note_text in note_text_list:
        caption = Title(
            text=note_text,
            align="left",
            text_font_size="4mm",
            text_font_style="italic",
        )
        fig_strk.add_layout(caption, "below")

    # Add the HoverTool to the figure
    fig_strk.add_tools(
        HoverTool(
            tooltips=tooltips,
            visible=False,
            formatters={"@Date": "datetime"},
        )
    )

    if table_output:
        print("")
        print("Print a table of streaks with +40 months or +10,000,000 jobs.")
        print("")
        print(
            strk_table_df[
                (
                    (strk_table_df["months_in_streak"] >= 40)
                    | (strk_table_df["total_emp_gain"] >= 10_000_000)
                )
            ][
                [
                    "strk_num",
                    "start_date",
                    "end_date",
                    "months_in_streak",
                    "total_emp_gain",
                    "avg_monthly_emp_gain",
                ]
            ].sort_values(
                by="months_in_streak", ascending=False
            )
        )

    if html_show:
        show(fig_strk)

    fig_lst = [fig_strk]

    if scatter_histogram:
        fig_title_scat_lst = [
            ("US employment streaks: consecutive positive monthly"),
            ("gains and average monthly employment gains, 1939 to 2024"),
        ]
        fig_title_scat_short = "Figure 2. US employment streaks scatterplot"
        filename_scat = "usempl_streaks_scatter" + end_date_str2 + ".html"
        output_file(
            os.path.join(image_dir, filename_scat),
            title=fig_title_scat_short,
            mode="inline",
        )

        # Format the tooltip
        tooltips_scat = [
            ("Start date", "@start_date"),
            ("End date", "@end_date"),
            ("Months in streak", "@months_in_streak{0.}"),
            ("Total employment gain", "@total_emp_gain{0,0.}"),
            ("Avg. monthly employment gain", "@avg_monthly_emp_gain{0,0.}"),
        ]

        min_avg_val = strk_table_df["avg_monthly_emp_gain"].min()
        max_avg_val = strk_table_df["avg_monthly_emp_gain"].max()

        strk_table_cds = ColumnDataSource(strk_table_df)
        datarange_avgmth_vals = max_avg_val - min_avg_val
        fig_scat = figure(
            height=500,
            width=700,
            x_axis_label="Streak length (months)",
            y_axis_label="Avg. monthly employment gains",
            x_minor_ticks=4,
            y_range=(
                min_avg_val - fig_buffer_pct * datarange_avgmth_vals,
                max_avg_val + fig_buffer_pct * datarange_avgmth_vals,
            ),
            y_minor_ticks=5,
            x_range=(
                (min_mth_val - fig_buffer_pct * datarange_mth_vals),
                121,
                # (max_mth_val + fig_buffer_pct * datarange_mth_vals),
            ),
            tools=[
                "save",
                "zoom_in",
                "zoom_out",
                "box_zoom",
                "pan",
                "undo",
                "redo",
                "reset",
                "help",
            ],
            toolbar_location="left",
        )
        fig_scat.toolbar.logo = None

        # Set title font size and axes font sizes
        fig_scat.xaxis.axis_label_text_font_size = "12pt"
        fig_scat.xaxis.major_label_text_font_size = "12pt"
        fig_scat.yaxis.axis_label_text_font_size = "12pt"
        fig_scat.yaxis.major_label_text_font_size = "12pt"

        # Reformat the labels for the ticks on the x and y axes
        y_tick_label_dict_scat = {
            0: "0",
            500_000: "500k",
            1_000_000: "1.0m",
            1_500_000: "1.5m",
            2_000_000: "2.0m",
        }

        fig_scat.yaxis.major_label_overrides = y_tick_label_dict_scat

        # Add scatter points for all streaks with months <40 and
        # total employment gain < 10 mil
        strk_table_other_cds = ColumnDataSource(
            strk_table_df[
                (strk_table_df["months_in_streak"] < 40)
                & (strk_table_df["total_emp_gain"] < 10_000_000)
            ]
        )
        strk_table_big_df = strk_table_df[
            (strk_table_df["months_in_streak"] >= 40)
            | (strk_table_df["total_emp_gain"] >= 10_000_000)
        ].sort_values(by="strk_num", ascending=True)
        strk_table_big_cds = ColumnDataSource(strk_table_big_df)

        for strk in range(strk_table_big_df.shape[0]):
            strk_table_i_cds = ColumnDataSource(strk_table_big_df.iloc[[strk]])
            scat_i = fig_scat.scatter(
                x="months_in_streak",
                y="avg_monthly_emp_gain",
                source=strk_table_i_cds,
                size=8,
                line_width=1,
                line_color="black",
                fill_color="marker_color",
                alpha=0.6,
                legend_label=(
                    strk_table_big_df.iloc[strk]["start_date"]
                    + " to "
                    + strk_table_big_df.iloc[strk]["end_date"]
                ),
            )

        scat_other = fig_scat.scatter(
            x="months_in_streak",
            y="avg_monthly_emp_gain",
            source=strk_table_other_cds,
            size=8,
            line_width=1,
            line_color="black",
            fill_color="marker_color",
            alpha=0.6,
            legend_label="All other streaks",
        )
        # Add legend
        fig_scat.legend.location = "top_right"
        fig_scat.legend.border_line_width = 2
        fig_scat.legend.border_line_color = "black"
        fig_scat.legend.border_line_alpha = 1
        fig_scat.legend.label_text_font_size = "4mm"
        fig_scat.legend.click_policy = "mute"

        # Add title and subtitle to the plot doing reverse loop through items
        # in fig_title_scat_lst
        for title_line_str in fig_title_scat_lst[::-1]:
            fig_scat.add_layout(
                Title(
                    text=title_line_str,
                    text_font_style="bold",
                    text_font_size="15pt",
                    align="center",
                ),
                "above",
            )

        # Add the HoverTool to the figure
        fig_scat.add_tools(
            HoverTool(
                tooltips=tooltips_scat,
                visible=False,
            )
        )

        # create the horizontal histogram
        hhist, hedges = np.histogram(
            strk_table_df["months_in_streak"], bins=24, range=(0, 120)
        )
        hzeros = np.zeros(len(hedges) - 1)
        hmax = max(hhist) * 1.15

        LINE_ARGS = dict(color="#3A5785", line_color=None)

        xhist = figure(
            toolbar_location=None,
            width=fig_scat.width,
            height=200,
            x_range=fig_scat.x_range,
            x_minor_ticks=4,
            y_range=(-3, hmax),
            y_minor_ticks=2,
            min_border=10,
            min_border_left=50,
            y_axis_location="right",
        )
        xhist.toolbar.logo = None
        xhist.xgrid.grid_line_color = None
        # xhist.yaxis.major_label_orientation = np.pi/4
        xhist.background_fill_color = "#fafafa"

        # Set title font size and axes font sizes
        xhist.xaxis.axis_label_text_font_size = "12pt"
        xhist.xaxis.major_label_text_font_size = "12pt"
        xhist.yaxis.axis_label_text_font_size = "10pt"
        xhist.yaxis.major_label_text_font_size = "10pt"

        xhist.quad(
            bottom=0,
            left=hedges[:-1],
            right=hedges[1:],
            top=hhist,
            color="blue",
            line_color="black",
        )
        hh1 = xhist.quad(
            bottom=0,
            left=hedges[:-1],
            right=hedges[1:],
            top=hzeros,
            alpha=0.5,
            **LINE_ARGS,
        )
        hh2 = xhist.quad(
            bottom=0,
            left=hedges[:-1],
            right=hedges[1:],
            top=hzeros,
            alpha=0.1,
            **LINE_ARGS,
        )

        # Create x-histogram labels
        xhist_label_source = ColumnDataSource(
            dict(
                bin_midpoints=(hedges[1:] - 2.5).tolist(),
                bin_heights=hhist.tolist(),
            )
        )

        xhist_labels = LabelSet(
            x="bin_midpoints",
            y="bin_heights",
            text="bin_heights",
            level="glyph",
            text_align="center",
            text_font_size="8pt",
            y_offset=3,
            source=xhist_label_source,
        )
        xhist.add_layout(xhist_labels)

        # create the vertical histogram
        vhist, vedges = np.histogram(
            strk_table_df["avg_monthly_emp_gain"],
            bins=18,
            range=(0, 1_800_000),
        )
        vzeros = np.zeros(len(vedges) - 1)
        vmax = max(vhist) * 1.2

        yhist = figure(
            toolbar_location=None,
            width=200,
            height=fig_scat.height,
            x_range=(-2, vmax),
            x_minor_ticks=2,
            y_range=fig_scat.y_range,
            y_minor_ticks=5,
            min_border=10,
            y_axis_location="right",
        )
        yhist.toolbar.logo = None
        yhist.ygrid.grid_line_color = None
        # yhist.xaxis.major_label_orientation = np.pi/4
        yhist.background_fill_color = "#fafafa"

        # Set title font size and axes font sizes
        yhist.xaxis.axis_label_text_font_size = "10pt"
        yhist.xaxis.major_label_text_font_size = "10pt"
        yhist.yaxis.axis_label_text_font_size = "12pt"
        yhist.yaxis.major_label_text_font_size = "12pt"

        # Reformat the labels for the ticks on the x and y axes
        yhist.yaxis.major_label_overrides = y_tick_label_dict_scat

        yhist.quad(
            left=0,
            bottom=vedges[:-1],
            top=vedges[1:],
            right=vhist,
            color="blue",
            line_color="black",
        )
        vh1 = yhist.quad(
            left=0,
            bottom=vedges[:-1],
            top=vedges[1:],
            right=vzeros,
            alpha=0.5,
            **LINE_ARGS,
        )
        vh2 = yhist.quad(
            left=0,
            bottom=vedges[:-1],
            top=vedges[1:],
            right=vzeros,
            alpha=0.1,
            **LINE_ARGS,
        )

        # Create y-histogram labels
        yhist_label_source = ColumnDataSource(
            dict(
                bin_midpoints=(vedges[1:] - 60_000).tolist(),
                bin_heights=vhist.tolist(),
            )
        )

        yhist_labels = LabelSet(
            x="bin_heights",
            y="bin_midpoints",
            text="bin_heights",
            level="glyph",
            text_align="left",
            text_font_size="8pt",
            x_offset=3,
            source=yhist_label_source,
        )
        yhist.add_layout(yhist_labels)

        layout = gridplot(
            [[fig_scat, yhist], [xhist, None]], merge_tools=False
        )
        layout.toolbar.logo = None

        # Add notes below image. The list note_text_list contains a tuple with
        # a string for every line of the notes
        updated_date_str = (
            download_date.strftime("%B")
            + " "
            + download_date.strftime("%d").lstrip("0")
            + ", "
            + download_date.strftime("%Y")
        )
        note_text_list = [
            (
                "Source: Richard W. Evans (@RickEcon), historical PAYEMS "
                + "data from FRED and BLS,"
            ),
            ("    updated " + updated_date_str + "."),
        ]
        for note_text in note_text_list:
            caption = Title(
                text=note_text,
                align="left",
                text_font_size="4mm",
                text_font_style="italic",
            )
            xhist.add_layout(caption, "below")

        # show(fig_scat)
        show(layout)

        # fig_lst.append(fig_scat)
        fig_lst.append(layout)

    return fig_lst, beg_date_str2, end_date_str2


# def usempl_streaks_hist():

#     return fig, beg_date_str2, end_date_str2


if __name__ == "__main__":
    # execute only if run as a script
    fig_lst, beg_date_str, end_date_str = usempl_streaks()



"""This file contains the usempl-plots package's metadata and dependencies."""

from setuptools import find_packages, setup

with open("README.md", "r") as readme_file:
    readme = readme_file.read()

setup(
    name="usempl-plots",
    version="0.0.5",
    author="Richard W. Evans",
    license="CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
    description="Package for creating plots of US employment and unemployment",
    long_description=readme,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: Developers",
        "License :: CC0 1.0 Universal (CC0 1.0) Public Domain Dedication",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Scientific/Engineering :: Information Analysis",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="Data analysis visualization tools US employment unemployment",
    url="https://github.com/OpenSourceEcon/usempl-plots",
    download_url="https://github.com/OpenSourceEcon/usempl-plots",
    project_urls={
        "Issue Tracker": "https://github.com/OpenSourceEcon/usempl-plots/issues",
    },
    packages=find_packages(),
    package_data={"usempl_plots": ["data/*"]},
    include_packages=True,
    python_requires=">=3.11",
    install_requires=[
        "numpy>=1.23.4",
        "scipy>=1.9.3",
        "pandas>=1.5.2",
        "pandas-datareader>=0.10.0",
        "bokeh>=3.0",
        "matplotlib",
        "pytest>=7.1.2",
        "pytest-cov",
        "coverage>=6.3.2",
        "codecov>=2.1.11",
        "black>=24.1.1",
        "pip>=22.3.1",
        "linecheck>=0.1.0",
    ],
    tests_require=["pytest"],
)



"""
This module plots the change in monthly US seasonally adjusted US nonfarm
employment (PAYEMS) by industry. The data is either taken from the St. Louis
Federal Reserve's FRED system(https://fred.stlouisfed.org/) or loads it from
this directory.

https://www.bls.gov/webapps/legacy/cesbtab1.htm

This module defines the following function(s):
    usempl_ind_chg()

The industries are:
| Industry                                 | Series ID     |  Sep 2003   |   Mar 2024  | Apr 2024 |
|------------------------------------------|---------------|-------------|-------------|----------|
- Total nonfarm:                             CES0000000001   130_252_000   158_133_000
  - Total private:                           CES0500000001   108_748_000   134_863_000
    - Goods Producing:                       CES0600000001    21_700_000    21_812_000
      - Mining and Logging                   CES1000000001       570_000       645_000
      - Construction                         CES2000000001     6_783_000     8_211_000
      - Manufacturing                        CES3000000001    14_347_000    12_956_000
    - Private Service Providing:             CES0800000001    87_048_000   113_051_000
      - Wholesale Trade                      CES4142000001     5_537_400     6_158_400
      - Retail Trade                         CES4200000001    14_911_800    15_659_900
      - Transportation and Warehousing       CES4300000001     4_162_100     6_540_500
      - Utilities                            CES4422000001       573_100       588_000
    - Information                            CES5000000001     3_162_000     3_017_000
    - Financial Activities                   CES5500000001     8_100_000     9_226_000
    - Professional and Business Services     CES6000000001    16_108_000    22_954_000
    - Private Education and Health Services: CES6500000001    16_883_000    26_101_000
      - Private Educational Services         CES6561000001     2_693_700     3_872_300
      - Health Care and Social Assistance    CES6562000001    14_189_400    22_228_200
    - Leisure and Hospitality                CES7000000001    12_208_000    16_905_000
    - Other Services                         CES8000000001     5_403_000     5_901_000
  - Government:                              CES9000000001    21_504_000    23_270_000
    - Federal Government                     CES9091000001     2_749_000     2_993_000
    - State Government                       CES9092000001     4_980_000     5_444_000
    - Local Government                       CES9093000001    13_775_000    14_833_000

Use this URL for API access: https://data.bls.gov/cgi-bin/srgate
"""

# Import packages
import numpy as np
import pandas as pd
import pandas_datareader as pddr
import datetime as dt
import os
from usempl_plots.get_payems import get_payems_data
from bokeh.io import output_file
from bokeh.plotting import curdoc, figure, show
from bokeh.models import ColumnDataSource, Title, Legend, HoverTool, LabelSet
from bokeh.layouts import gridplot

# from bokeh.models import Label
from bokeh.palettes import Category20, Plasma256, Viridis8

"""
Define functions
"""


def usempl_ind_chg(
    start_date="2003-09-01",
    end_date="max",
    download_from_internet=True,
    fig_title_strk=(
        "Change in US employment my industry: Sep. 2003 to Apr. 2024"
    ),
    html_show=True,
):
    """
    This function creates the HTML and JavaScript code for the dynamic
    visualization of the US change in monthly seasonally adjusted nonfarm
    employment (PAYEMS) by industry.

    Args:
        start_date (str): start date of PAYEMS time series in 'YYYY-mm-dd'
            format or 'min'
    """
    # Create data and images directory paths
    cur_path = os.path.split(os.path.abspath(__file__))[0]
    image_dir = os.path.join(cur_path, "images")
    data_dir = os.path.join(cur_path, "data")

    data_df = pd.read_csv(
        os.path.join(data_dir, "industry", "jobs_by_industry.csv")
    )
    data_df["diff_Sep03_Mar24"] = data_df["Mar24"] - data_df["Sep03"]
    data_df["pctchg_Sep03_Mar24"] = (
        (data_df["Mar24"] - data_df["Sep03"]) / data_df["Sep03"]
    ) * 100

    print(
        "Total jobs created from Sep. 2003 to Mar. 2024: ",
        int(
            data_df.loc[
                data_df["Industry"] == "Total nonfarm", "diff_Sep03_Mar24"
            ]
        ),
    )
    print("")
    print(
        "Percent change in jobs from Sep. 2003 to Mar. 2024:",
        data_df.loc[
            data_df["Industry"] == "Total nonfarm", "pctchg_Sep03_Mar24"
        ],
    )
    print("")
    print(
        data_df[
            [
                "Industry",
                "Sep03",
                "Mar24",
                "diff_Sep03_Mar24",
                "pctchg_Sep03_Mar24",
            ]
        ]
    )

    return data_df


if __name__ == "__main__":
    # execute only if run as a script
    data_df = usempl_ind_chg()
